function [X,Y] = contrast(address,h,w)
//
// Algorithm Contrast Technique
// Developed by: Junio Moreira / Roberto Alves Braga Junior
// Federal University of Lavras - UFLA
// =================================================
// BSLD (Library Speckle Laser Dynamic) version 1.0
// Date: september of 2013
// Review: october of 2013
// Software Register in Natinal Institute of Industrial Property INPI
// Registration number “Bibliotecas Speckle Laser Dinâmico para Scilab e Matlab” BR 51 2014 000018 4
// Type of Registration: Software free to use (Library free to use)
// This library is totally free to use in education, research and other noncommercial activities.
//
// If you use it, please, cite it properly:
// Moreira, J, Cardoso, RR, Braga, RA 2014 Quality test protocol to dynamic laser speckle analysis. 
// Optics and Lasers in Engineering. 10.1016/j.optlaseng.2014.04.005
//
// =================================================
// Main Program
//==================================================
// In Scilab 5.4 you have to install the libraries SPD, SIVP, PLOTLIB
// 
//  Input:
//  address: directory of the images
//  h, w: height and width of the subwindows where the contrast will be evaluated
//  Output:
//   Y is the Constrast image
//   X is the value of the contrast
//  
//-----------------------------------------------------------------------------//
//
// Clear memory
    clc();
    clear all;

// Set free the memory
    stacksize('max');    

// Start Time
    initiationOfTheClock = getdate();    
    
// Set the directory of the Images    
    cd(address);           

// Identification of the images -- see the format of the image with only numbers.
    DATA = imread ('1.bmp');
    a = size(DATA);    
    line = a(1,1);
    column = a(1,2);
    preview = zeros(line,column);
    
//-----------------------------------------------------------------------------//    
//        
// Processing       
        
    clc();
    disp('Start CONSTRAST method...');
    
    a = size(DATA);
    Y = zeros(a(1,1),a(1,2));    
    Z = zeros(h,w);
    contraste = zeros(floor(a(1,1)/h),floor(a(1,2)/w));
    x = 1;
    y = 1;
    preview(:,:) = DATA (:,:);
        
    for b = 1:h:a(1,1)-h
        for c =1:w:a(1,2)-w+1
            Z(:,:) = DATA(b:b+h-1,c:c+w-1);
            //Y(b:b+h-1,c:c+w-1) = (std2(Z)/(mean2(Z)+1));   
            Y(b:b+h-1,c:c+w-1) = 1 - (std2(Z)/(mean2(Z)+1));        
            contraste(x,y) = 1 - (std2(Z)/(mean2(Z)+1));
            y = y+1;  
        end       
        x = x+1;
        y = 1; 
    end
    
// Directory Functions    
    //cd('C:\bsld\fsbl\');
    
// Porcentage of method Saturation
    disp('Final Results Porcentage method Contraste: ');      
    disp('Matriz of value Contrast --> ');    
    disp(contraste);
        
// Calculating the contrast final    
    disp('Final Results Constrast method: ');
    X = mean2(Y);     
    disp('Value of Contrast --> ');    
    disp(X);    
    
//Plotting graphs    
    scf(0);
    clf(0);         
    colormap gray;       
    imagesc(Y(:,:));                  
    title('Constrast Method!');  
    colorbar;
    axis equal;
    
    scf(1);
    clf(1);             
    //colormap gray;
    imagesc(preview(:,:));    
    title('Image Preview:');   
    axis equal;    
       
// Recording results
    gravar = address + 'contrast_scilab.bmp';
    imwrite(Y,gravar);
    
// Training time
    stopOfTheClock = getdate(); //stops the clock
    etr = etime(stopOfTheClock,initiationOfTheClock); //calculates the time elapsed
    disp('');
    disp('Training time: ' + string(etr/60) + ' minutes.');    
    disp('ENDED!!!');
    sleep(5000);
    
// =============================================================================
endfunction
