function [S,saturacao,dark] = sat(WINDOW,h,w)
//
//
// Algorithm Saturation Technique
// Developed by: Junio Moreira / Roberto Alves Braga Junior
// Federal University of Lavras - UFLA
// =================================================
//
// In:  RABAL, H. J.; BRAGA, R. A. Dynamic laser speckle and applications. Boca
//      Raton: CRC, 2008. 304 p.
//
//  Output:
//  Y are the value of the calculating areas of dark or saturated window analysis
//  
// =================================================
// Main Program
//==================================================
// In Scilab 5.4 you have to install the libraries SPD, SIVP, PLOTLIB
// BSLD (Library Speckle Laser Dynamic) version 1.0
// Date: september of 2013
// Review: october of 2013
// Software Register in Natinal Institute of Industrial Property INPI
// Registration number “Bibliotecas Speckle Laser Dinâmico para Scilab e Matlab” BR 51 2014 000018 4
// Type of Registration: Software free to use (Library free to use)
// This library is totally free to use in education, research and other noncommercial activities.
//
// If you use it, please, cite it properly:
// Moreira, J, Cardoso, RR, Braga, RA 2014 Quality test protocol to dynamic laser speckle analysis. 
// Optics and Lasers in Engineering. 10.1016/j.optlaseng.2014.04.005
//
// The Saturation adopts some functions that follow
//-----------------------------------------------------------------------------//
//
// Algorithm Saturation Technique
//  Input:
//  DATA: participant image under evaluation
//  h, w: height and width of the subwindows where the contrast will be evaluated
//  
//  Output:
//  Y are the value of the calculating areas of dark or saturated window analysis
//
// =============================================================================
// Begin
//    
    a = size(WINDOW);
    S = zeros(a(1,1),a(1,2));
    P = 0.5;
    MES = 0;
    MEI = 15;
    
    saturacao = 0;
    dark = 0;        
    
    for b = 1:a(1,1)
        for c = 1:a(1,2)
            
            if (WINDOW(b,c) >= (255 - MES)) 
                saturacao = saturacao+1;                               
            end
            
            if (WINDOW(b,c) <= (1 + MEI)) 
                dark = dark+1;
            end            
                        
        end        
    end         
    
    if (saturacao >= (h*w*P)) then
        S(:,:) = 255;        
    end
    
    if (dark >= (h*w*P)) then
        S(:,:) = 0;        
    end 
        
    if ((saturacao < (h*w*P)) & (dark < (h*w*P))) then
        S(:,:) = WINDOW(:,:);        
    end   
    
    if ((saturacao >= (h*w*P)) & (dark >= (h*w*P))) then              
        J = ones(a(1,1), a(1,2));
        L = zeros(a(1,1), a(1,2));
        J = tril(J,0);
        L = triu(L,0);
        S = J+L;
    end   
    
endfunction

// This is the main function that you have to run
function [Y] = saturation(address,h,w)
//
// Algorithm Saturation Technique
// Developed by: Junio Moreira / Roberto Alves Braga Junior
// Federal University of Lavras - UFLA
// =================================================
//
// In:  RABAL, H. J.; BRAGA, R. A. Dynamic laser speckle and applications. Boca
//      Raton: CRC, 2008. 304 p.
//
//  Output:
//  Y are the value of the calculating areas of dark or saturated window analysis
//  
// =================================================
// Main Program
//==================================================
// In Scilab 5.4 you have to install the libraries SPD, SIVP, PLOTLIB
// BSLD (Library Speckle Laser Dynamic) version 1.0
// Date: september of 2013
// Review: october of 2013
// Software Register in Natinal Institute of Industrial Property INPI
// Registration number “Bibliotecas Speckle Laser Dinâmico para Scilab e Matlab” BR 51 2014 000018 4
// Type of Registration: Software free to use (Library free to use)
// This library is totally free to use in education, research and other noncommercial activities.
//
// If you use it, please, cite it properly:
// Moreira, J, Cardoso, RR, Braga, RA 2014 Quality test protocol to dynamic laser speckle analysis. 
// Optics and Lasers in Engineering. 10.1016/j.optlaseng.2014.04.005
//
//-----------------------------------------------------------------------------//
//
// Clear memory
    clc();
    clear all;

// Set free the memory
    stacksize('max');    

// Start Time
    initiationOfTheClock = getdate();    
    
// Set the directory of the Images    
    cd(address);           

// Identification of the images -- see the format of the image with only numbers.
    DATA = imread ('1.bmp');
    a = size(DATA);    
    line = a(1,1);
    column = a(1,2);
    preview = zeros(line,column);
    
//-----------------------------------------------------------------------------//    
//        
// Processing       
        
    a = size(DATA);     
    Y = zeros(a(1,1),a(1,2));    
    Z = zeros(h,w);     
    MP_sat = zeros(a(1,1)/h,a(1,2)/w);
    MP_dark = zeros(a(1,1)/h,a(1,2)/w);
    x = 1;
    y = 1;
    preview(:,:) = DATA(:,:);
    
    clc();
    disp('Start method SATURATION...');
    
    for b = 1:h:a(1,1)-h
        for c =1:w:a(1,2)-w+1
            Z(:,:) = DATA(b:b+h-1,c:c+w-1);                   
            [d,saturacao,dark] = sat(Z,h,w);            
            Y(b:b+h-1,c:c+w-1) = d(:,:);                      
            MP_sat(x,y) = ((100*saturacao)/(h*w));
            MP_dark(x,y) = ((100*dark)/(h*w));            
            y = y+1;
        end
        x = x+1;
        y = 1;
    end 
    
// Directory Functions    
    //cd('C:\bsld\fsbl\');
    
// Porcentage of method Saturation
    disp('Final Results Porcentage method Saturation: ');      
    disp('Matriz de Porcentage Saturation --> ');    
    disp(MP_sat);
    
    disp('Final Results Porcentage method Saturation: ');      
    disp('Matriz de Porcentage Area Dark  --> ');    
    disp(MP_dark);    
    
//Plotting graphs    
    scf(0);
    clf(0);         
    colormap gray;
    imagesc(preview(:,:));    
    title('Image Preview:');   
    axis equal;
    
    scf(1);
    clf(1);    
    //colormap gray;
    imagesc(Y(:,:));   
    axis equal;     
    xtitle('Saturation Method -->','Área Saturada - Vermelho','Área Escura - Azul' );
    title('Saturation Method:');  
    
// Recording results
    Y = uint8(Y);          
    gravar = address + 'saturation_scilab.bmp';
    imwrite(Y,gravar);
    
// Training time
    stopOfTheClock = getdate(); //stops the clock
    etr = etime(stopOfTheClock,initiationOfTheClock); //calculates the time elapsed
    disp('');
    disp('Training time: ' + string(etr/60) + ' minutes.');    
    disp('ENDED!!!');
    sleep(5000);
    
// =============================================================================
endfunction
